 /****************************************************************
 *
 __LICENSE_GO_HERE__
 *
 * Copyright (c) 2010 Nokia
 *
 * File name   : aegis-session.h
 * Author      : Vesa Jääskeläinen
 *               (vesa.jaaskelainen@nokia.com)
 ****************************************************************/
#ifndef AEGIS_SESSION_H_
#define AEGIS_SESSION_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/resource.h>

typedef enum
{
	EnvironmentVariable = 0,
	UlimitResourceLimit = 1
} AegisSessionEntryType;

typedef struct _AegisSessionEntry
{
	struct _AegisSessionEntry *next;

	AegisSessionEntryType type;

	char *name;
	char *value;
} AegisSessionEntry;

#define AEGIS_SESSION_MODE_READ_ONLY	0
#define AEGIS_SESSION_MODE_READ_WRITE	1
#define AEGIS_SESSION_MODE_CLEAR		2

/**
 * Open session & load session data.
 *
 * @param dataSet Pointer to session entries list where to add new entry.
 * @param flags Whether to open file in read/write mode (AEGIS_SESSION_MODE_READ_WRITE),
 *              clearing mode (AEGIS_SESSION_MODE_CLEAR) or as read only
 *              (AEGIS_SESSION_MODE_READ_ONLY).
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionOpen(AegisSessionEntry **dataSet, int mode);

/**
 * Close session.
 *
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionClose(void);

/**
 * Save session data. If file is locked call will block until it can be written.
 *
 * @credential libaegis-session::aegis-session-data In order to save signed data this credential is needed.
 * @param dataSet Session entries to be saved. NULL causes session data to be cleared.
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionSaveData(AegisSessionEntry *dataSet);

/**
 * Free session data allocated to memory.
 *
 * @param dataSet Session entries to be freed. NULL safe.
 */
void aegisSessionFreeData(AegisSessionEntry *dataSet);

/**
 * Add new setting or replace existing entry with same name.
 *
 * @param dataSet Pointer to session entries list where to add new entry.
 * @param type Session entry type.
 * @param name Session entry name.
 * @param value Session entry value.
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionSetSetting(AegisSessionEntry **dataSet, AegisSessionEntryType type, const char *name, const char *value);

/**
 * Remove setting from session.
 *
 * @param dataSet Pointer to session entries list from where to remove entry.
 * @param name Session entry name to be removed.
 */
void aegisSessionRemoveSetting(AegisSessionEntry **dataSet, const char *name);

/**
 * Translates ulimit value from string form to real limit values.
 *
 * Only forms "num", "num:num" are supported. Other values generated failure.
 *
 * @param entry From which entry to read ulimit values.
 * @param softLimit Pointer where to store decoded soft limit value.
 * @param hardLimit Pointer where to store decoded hard limit value.
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionTranslateUlimit(const AegisSessionEntry *entry, rlim_t *softLimit, rlim_t *hardLimit);

/**
 * Helper to aid in configuring of environment variables for applications.
 *
 * Actions performed:
 * - Opens session file for reading
 * - Verifies integrity of session file
 * - Configures environment variables for current process
 * - Closes session file.
 *
 * Note: This method should not be called when doing other manipulation for Aegis Session
 * data or the Aegis Session file access has been opened with aegisSessionOpen().
 * If session file is already open this function call will fail.
 *
 * @return Non-zero on success, zero on failure.
 */
int aegisSessionSetupEnvironmentHelper(void);

#ifdef __cplusplus
}
#endif

#endif /* AEGIS_SESSION_H_ */
